/*
 *  bld_codes.c
 */

/*
 *    SAM-QFS_notice_begin
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at pkg/OPENSOLARIS.LICENSE
 * or https://illumos.org/license/CDDL.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at pkg/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 *    SAM-QFS_notice_end
 */

#pragma ident "$Revision: 1.40 $"


#define	MAIN
#define	DEC_INIT

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include "sam/types.h"
#include "sam/param.h"
#include "aml/device.h"
#include "sam/devinfo.h"
#include "aml/scsi_error.h"
#include "sam/devinfo.h"


sam_scsi_err_t    acl452[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x00, MAN_INTER, DOWN_EQU, /* Major problem */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x5a, 0x01, 0x00, WAIT_READY,
	0x02, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x07, STOP_BUTTON, WAIT_READY,
	0x02, 0x80, 0x09, OFFLINE, WAIT_READY,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,   /* Import element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x06, 0x80, 0x07, STOP_BUTTON, WAIT_READY,
	0x06, 0x80, 0x08, OFFLINE, WAIT_READY,
	0x06, 0x80, 0x09, STANDBY, WAIT_READY,
	0xff, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    acl2640[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x00, MAN_INTER, DOWN_EQU, /* Major problem */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,
	0x02, 0x5a, 0x01, 0x00, WAIT_READY,
	0x02, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x07, STOP_BUTTON, WAIT_READY,
	0x02, 0x80, 0x09, OFFLINE, WAIT_READY,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,   /* Import element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x06, 0x80, 0x07, STOP_BUTTON, WAIT_READY,
	0x06, 0x80, 0x08, OFFLINE, WAIT_READY,
	0x06, 0x80, 0x09, STANDBY, WAIT_READY,
	0x0b, 0x8c, 0x07, 0x00, NO_MEDIA, /* Cartridge incorrectly loaded */
	0xff, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    atlp3000[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x00, MAN_INTER, DOWN_EQU, /* Major problem */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,
	0x02, 0x5a, 0x01, 0x00, WAIT_READY,
	0x02, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x07, STOP_BUTTON, WAIT_READY,
	0x02, 0x80, 0x09, OFFLINE, WAIT_READY,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,   /* Import element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x06, 0x80, 0x07, STOP_BUTTON, WAIT_READY,
	0x06, 0x80, 0x08, OFFLINE, WAIT_READY,
	0x06, 0x80, 0x09, STANDBY, WAIT_READY,
	0xff, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t	 atl1500[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x3E, 0x00, 0x00, WAIT_READY,
	0x02, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x01, 0x00, WAIT_READY, 	/* OCP in menu mode */
	0x04, 0x00, 0x00, 0x00, IGNORE,
	0x04,	0x15, 0x01, HARDWARE, DOWN_EQU,
	0x05, 0x15, 0x01, HARDWARE, DOWN_EQU,
	0x05, 0x1A, 0x00, 0x00, ILLREQ,
	0x05, 0x20, 0x00, 0x00, ILLREQ,
	0x05, 0x24, 0x00, 0x00, ILLREQ,
	0x05, 0x25, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x02, 0x00, ILLREQ,
	0x05, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x05, 0x39, 0x00, 0x00, ILLREQ,
	0x05, 0x3B, 0x0D, 0x00, D_ELE_FULL,
	0x05, 0x3B, 0x0E, 0x00, S_ELE_EMPTY,
	0x05, 0x83, 0x10, HARDWARE, DOWN_EQU,
	0x06, 0x28, 0x00, 0x00, WAIT_READY,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2A, 0x00, WRONG_MODE, DOWN_EQU, /* Mode parameters changed */
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t	 odi_neo[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x3E, 0x00, 0x00, WAIT_READY,
	0x02, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x01, 0x00, WAIT_READY, 	/* OCP in menu mode */
	0x04, 0x00, 0x00, 0x00, IGNORE,
	0x04,	0x15, 0x01, HARDWARE, DOWN_EQU,
	0x05, 0x15, 0x01, HARDWARE, DOWN_EQU,
	0x05, 0x1A, 0x00, 0x00, ILLREQ,
	0x05, 0x20, 0x00, 0x00, ILLREQ,
	0x05, 0x24, 0x00, 0x00, ILLREQ,
	0x05, 0x25, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x02, 0x00, ILLREQ,
	0x05, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x05, 0x39, 0x00, 0x00, ILLREQ,
	0x05, 0x3B, 0x0D, 0x00, D_ELE_FULL,
	0x05, 0x3B, 0x0E, 0x00, S_ELE_EMPTY,
	0x05, 0x83, 0x10, HARDWARE, DOWN_EQU,
	0x06, 0x28, 0x00, 0x00, WAIT_READY,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2A, 0x00, WRONG_MODE, DOWN_EQU, /* Mode parameters changed */
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};


sam_scsi_err_t	hp_c7200[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0x00, 0x00, 0x00, IGNORE,		/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,	/* becoming ready */
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,	/* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x02, 0x3a, 0x02, DOOR_OPEN, WAIT_INIT,
	0x02, 0x3a, 0x80, DOOR_OPEN, WAIT_INIT,
	0x04, 0x42, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,	/* power on  */
	0x06, 0x3f, 0x00, 0x00, WAIT_READY,	/* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   hpoplib[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0x00, 0x00, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,    /* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x04, 0x42, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x00, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   dlt2700[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0x00, 0x00, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,    /* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x04, 0x42, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x06, 0x53, 0x02, STANDBY, WAIT_READY_LONG,
	0x06, 0x29, 0x00, 0x00, LOG_IGNORE,   /* reset  */
	0x06, 0x3f, 0x00, 0x00, WAIT_READY,   /* microcode changed */
	0x06, 0x5a, 0x00, 0x00, WAIT_READY,   /* operator request */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    metd360[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x04, 0x08, 0x00, 0x00, LONG_WAIT_LOG, /* communication failure */
	0x04, 0x15, 0x87, PICKFAIL, CLR_TRANS_RET,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x00, 0x00, WAIT_READY,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,   /* Import element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x01, 0x00, IGNORE,	/* Mode parameters changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x06, 0x82, 0x00, 0x00, REZERO,	/* barcode misread */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    metd28[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x00, 0x00, WAIT_READY,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,   /* Import element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x01, 0x00, IGNORE,	/* Mode parameters changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   docstor[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, REZERO,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x82, DOOR_OPEN, LONG_WAIT_LOG,
	0x02, 0x80, 0x2c, DOOR_OPEN, LONG_WAIT_LOG,
	0x02, 0x80, 0x04, 0x00, WAIT_READY,
	0x04, 0x08, 0x00, 0x00, CLR_TRANS_RET,
	0x04, 0x42, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x29, 0x80, POWER_ON, WAIT_INIT,
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    exb210[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_INIT,
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0x84, 0x00, WAIT_INIT,
	0x02, 0x04, 0x89, WRONG_MODE, DOWN_EQU,
	0x02, 0x04, 0x8d, WRONG_MODE, DOWN_EQU,
	0x02, 0x04, 0x8e, WRONG_MODE, DOWN_EQU,
	0x04, 0x15, 0xff, MAN_INTER, DOWN_EQU,
	0x04, 0x40, 0x01, MAN_INTER, DOWN_EQU,
	0x04, 0x40, 0x88, DOOR_OPEN, WAIT_INIT,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x53, 0x02, 0x00, LOG_IGNORE,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x00, 0x00, WAIT_INIT,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x01, 0x00, IGNORE,	/* Mode parameters changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    exbx80[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_INIT,
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0x89, WRONG_MODE, DOWN_EQU,
	0x02, 0x04, 0x8d, WRONG_MODE, DOWN_EQU,
	0x04, 0x15, 0xff, MAN_INTER, DOWN_EQU,
	0x04, 0x40, 0x88, DOOR_OPEN, WAIT_INIT,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x53, 0x02, 0x00, LOG_IGNORE,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x00, 0x00, WAIT_INIT,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x01, 0x00, IGNORE,	/* Mode parameters changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    sonydms[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,	/* Initializing */
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,	/* Initial Required */
	0x02, 0x04, 0x03, 0x00, WAIT_INIT,	/* Recover Required */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0x84, 0x00, WAIT_INIT,	/* Emergency Stop */
	0x02, 0x04, 0x8d, WRONG_MODE, DOWN_EQU, /* Test mode */
	0x02, 0x04, 0x8e, WRONG_MODE, DOWN_EQU, /* Sequential Mode */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x5a, 0x00, 0x00, IGNORE,	 /* Needs formatting */
	0x06, 0x28, 0x00, 0x00, WAIT_INIT,
	0x06, 0x28, 0x8d, 0x00, IGNORE,	 /* Test mode is canceled */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x01, 0x00, IGNORE,	 /* Mode parameters changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,	/* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    sonycsm[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x80, 0x00, WAIT_READY,
	0x02, 0x80, 0x71, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x7b, DOOR_OPEN, WAIT_INIT,
	0x04, 0x02, 0x00, HARDWARE, DOWN_EQU,
	0x04, 0x15, 0x01, HARDWARE, DOWN_EQU,
	0x04, 0x15, 0x80, HARDWARE, DOWN_EQU,
	0x04, 0x15, 0x81, HARDWARE, DOWN_EQU,
	0x04, 0x44, 0x00, HARDWARE, DOWN_EQU,
	0x04, 0x53, 0x00, HARDWARE, DOWN_EQU,
	0x04, 0x80, 0x18, HARDWARE, DOWN_EQU,
	0x04, 0x80, 0x19, HARDWARE, DOWN_EQU,
	0x04, 0x80, 0x79, HARDWARE, DOWN_EQU,
	0x04, 0x80, 0xa0, HARDWARE, DOWN_EQU,
	0x04, 0x90, 0x14, HARDWARE, DOWN_EQU,
	0x04, 0x90, 0x15, HARDWARE, DOWN_EQU,
	0x04, 0x91, 0x20, HARDWARE, DOWN_EQU,
	0x04, 0x91, 0x23, HARDWARE, DOWN_EQU,
	0x04, 0x91, 0x30, HARDWARE, DOWN_EQU,
	0x04, 0x91, 0x32, HARDWARE, DOWN_EQU,
	0x04, 0x92, 0x40, HARDWARE, DOWN_EQU,
	0x04, 0x92, 0x50, HARDWARE, DOWN_EQU,
	0x04, 0x93, 0xe9, HARDWARE, DOWN_EQU,
	0x04, 0x94, 0xee, HARDWARE, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x1a, 0x00, 0x00, ILLREQ,
	0x05, 0x20, 0x00, 0x00, ILLREQ,
	0x05, 0x21, 0x01, 0x00, ILLREQ,
	0x05, 0x24, 0x00, 0x00, ILLREQ,
	0x05, 0x25, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x01, 0x00, ILLREQ,
	0x05, 0x26, 0x02, 0x00, ILLREQ,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x80, 0x00, 0x00, ILLREQ,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x90, 0x00, IGNORE,
	0x06, 0x5b, 0x01, 0x00, IGNORE,
	0x06, 0x5b, 0x02, 0x00, IGNORE,
	0x06, 0xef, 0xfc, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   speclog[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0x84, DOOR_OPEN, WAIT_INIT,
	0x04, 0x4c, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0x85, 0x01, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x0c, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x0e, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x0f, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x18, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x19, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x1c, 0x00, WAIT_INIT,
	0x04, 0x85, 0x21, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x25, MAN_INTER, DOWN_EQU,
	0x04, 0x85, 0x35, 0x00, WAIT_INIT,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x2a, 0x01, 0x00, IGNORE,
	0x06, 0xff, 0xff, 0x00, LOG_IGNORE,
	0x09, 0x13, 0x00, 0x00, BAD_BARCODE,
	0x09, 0x83, 0x01, 0x00, BAD_BARCODE,
	0x09, 0x83, 0x02, MEDIA_RESER, MARK_UNAVAIL,
	0x09, 0x83, 0x11, 0x00, IGNORE,
	0x09, 0x83, 0x12, 0x00, IGNORE,
	0x09, 0x84, 0x08, 0x00, WAIT_READY,
	0x09, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   qual82xx[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x82, DOOR_OPEN, WAIT_READY, /* I/O port extended */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0x85, DOOR_OPEN, WAIT_INIT,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x00, 0x00, LOG_IGNORE,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,  /* I/E element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0xff, 0xff, 0x00, LOG_IGNORE,
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t	quantumc4[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, DOWN_EQU,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x3E, 0x00, 0x00, WAIT_READY,
	0x02, 0x80, 0x00, DOOR_OPEN, WAIT_INIT,
	0x02, 0x80, 0x01, 0x00, WAIT_READY,   /* OCP in menu mode */
	0x02, 0x80, 0x02, DOOR_OPEN, WAIT_READY,
	0x04, 0x00, 0x00, 0x00, IGNORE,
	0x04, 0x15, 0x01, HARDWARE, DOWN_EQU,
	0x05, 0x1A, 0x00, 0x00, ILLREQ,
	0x05, 0x20, 0x00, 0x00, ILLREQ,
	0x05, 0x24, 0x00, 0x00, ILLREQ,
	0x05, 0x25, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x00, 0x00, ILLREQ,
	0x05, 0x26, 0x02, 0x00, ILLREQ,
	0x05, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x05, 0x39, 0x00, 0x00, ILLREQ,
	0x05, 0x3B, 0x0D, 0x00, D_ELE_FULL,
	0x05, 0x3B, 0x0E, 0x00, S_ELE_EMPTY,
	0x05, 0x3B, 0x80, PICKFAIL, NO_MEDIA,  /* Transport element full */
	0x05, 0x83, 0x10, HARDWARE, DOWN_EQU,
	0x06, 0x28, 0x00, 0x00, WAIT_READY,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE, /* Import/Export element accessed */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2A, 0x01, WRONG_MODE, DOWN_EQU, /* Mode parameters changed */
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   adic448[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, REZERO,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x04, 0x08, 0x00, 0x00, CLR_TRANS_RET,
	0x04, 0x42, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   adic100[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0xff, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x53, 0x81, DOOR_OPEN, WAIT_INIT,
	0x05, 0x83, 0x03, 0x00, BAD_BARCODE,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   adic1000[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_INIT,
	0x02, 0x04, 0xff, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x30, 0x00, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x53, 0x81, DOOR_OPEN, WAIT_INIT,
	0x05, 0x83, 0x02, 0x00, BAD_BARCODE,
	0x05, 0x83, 0x03, 0x00, BAD_BARCODE,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* scsi parity error */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   stk97xx[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, REZERO,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x30, 0x00, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* scsi parity error */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   ibm3570c[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, REZERO,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x85, DOOR_OPEN, WAIT_READY_LONG,
	0x02, 0x04, 0x8d, WRONG_MODE, LONG_WAIT_LOG,
	0x02, 0x04, 0x8e, WRONG_MODE, LONG_WAIT_LOG,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0xff, 0x00, IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* scsi parity error */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t    cyg1803[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU,
	0x04, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* manual say 04,should be 02? */
	0x04, 0x80, 0x82, DOOR_OPEN, WAIT_INIT,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x80, 0x40, 0x00, CLR_TRANS_RET,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x28, 0x01, 0x00, LOG_IGNORE,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x5a, 0x00, POWER_ON, WAIT_INIT,
	0x06, 0x80, 0x21, 0x00, IGNORE,
	0x06, 0xFF, 0xFF, 0x00, DOWN_EQU,
	0x0B, 0x80, 0x80, 0x00, LOG_IGNORE,
	0x0B, 0x80, 0x81, 0x00, LOG_IGNORE,
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   plasmond[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0x00, 0x00, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,    /* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x04, 0x42, 0x00, SELF_FAILED, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA,
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x52, 0x5a, INCOMPATIBLE, ILLREQ,
	0x05, 0x52, 0x60, INCOMPATIBLE, ILLREQ,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x00, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   plasmong[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0xff, 0xff, 0x00, IGNORE,	/* Recovered error */
	0x02, 0x04, 0xff, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0xff, 0xff, 0x00, WAIT_READY,   /* becoming ready */
	0x04, 0x52, 0x59, INCOMPATIBLE, MARK_UNAVAIL,
	0x04, 0x80, 0x1a, HARDWARE, DOWN_EQU,
	0x04, 0x80, 0x1d, HARDWARE, NO_MEDIA,
	0x04, 0x80, 0x1e, HARDWARE, NO_MEDIA,
	0x04, 0x80, 0x23, HARDWARE, DOWN_SUB_EQU,
	0x04, 0x80, 0x24, HARDWARE, DOWN_SUB_EQU,
	0x04, 0x80, 0x25, HARDWARE, DOWN_SUB_EQU,
	0x04, 0x80, 0x26, HARDWARE, DOWN_SUB_EQU,
	0x04, 0x80, 0x80, DOOR_OPEN, WAIT_READY_LONG,
	0x04, 0x80, 0x81, DOOR_OPEN, WAIT_READY_LONG,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x3b, 0x0d, HARDWARE, D_ELE_FULL,
	0x05, 0x3b, 0x0e, HARDWARE, S_ELE_EMPTY,
	0x05, 0x52, 0x5a, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0x52, 0x60, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x00, 0x00, WAIT_READY,   /* microcode changed */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   stklxx[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x81, WRONG_MODE, DOWN_EQU, /* Operator put unit in */
						/* maintenance mode */
	0x02, 0x3a, 0x02, DOOR_OPEN, WAIT_READY, /* CAP is open */
	0x02, 0xff, 0xff, 0x00, WAIT_READY,   /* becoming ready */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x30, 0x00, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA, /* Medium not present */
						/* drive not unloaded */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL, /* Destination element full */
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,  /* Source element empty */
	0x05, 0xff, 0xff, 0x00, ILLREQ, /* Catch-all illegal request */
	0x06, 0x28, 0xff, 0x00, IGNORE, /* Not ready-to-ready transition */
					/* CAP element accessed */
	0x06, 0x29, 0x01, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* SCSI parity error */
	0x0b, 0x48, 0x00, 0x00, LONG_WAIT_LOG, /* Initiator detected error */
	0x0b, 0x4e, 0x00, 0x00, LONG_WAIT_LOG, /* Command overlap error */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

/* SCSI Refrence Guide: Request Sense 03, Additional Sense Codes ans Qualifiers
   0h = No Sense, indicating a successful command.
   2h = Not Ready
   3h = Medium Error
   4h = Hardware Error
   5h = Illegal Request
   6h = Unit Attention
   Bh = Aborted Command
   FFh catch all
   Sense Key, ASC, ASCQ, Description, SAM reaction 
*/
sam_scsi_err_t   stksl3000[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,

	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x81, WRONG_MODE, DOWN_EQU, /* Operator put unit in */
						/* maintenance mode */
	0x02, 0x3a, 0x02, DOOR_OPEN, WAIT_READY, /* CAP is open */
	0x02, 0x74, 0x71, ACCESS_DENIED, DOWN_EQU, /* Partition Access Denied */
	0x02, 0xff, 0xff, 0x00, WAIT_READY,   /* becoming ready */

	0x04, 0x40, 0x01, HARDWARE, DOWN_EQU, /* General hardware errors */
	0x04, 0x40, 0x02, HARDWARE, DOWN_EQU, /* Drive hardware errors */
	0x04, 0x40, 0x03, HARDWARE, DOWN_EQU, /* CAP hardware errors */
	0x04, 0x44, 0x00, HARDWARE, DOWN_EQU, /* Embedded SW errors */
	0x04, 0x53, 0x00, HARDWARE, LOG_IGNORE, /* Load/Eject failed */

	0x05, 0x30, 0x00, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA, /* Medium not present */
						/* drive not unloaded */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL, /* Destination element full */
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,  /* Source element empty */
	0x05, 0xff, 0xff, 0x00, ILLREQ, /* Catch-all illegal request */

	0x06, 0x28, 0xff, 0x00, IGNORE, /* Not ready-to-ready transition */
					/* CAP element accessed */
	0x06, 0x29, 0x01, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* not for SL4000: microcode changed */

	0x09, 0x3b, 0x1a, DRVSLOT_EMPTY, MARK_EMPTY,	/* SL4000, Drive Slot empty */

	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* not for SL4000: SCSI parity error */
	0x0b, 0x48, 0x00, 0x00, LONG_WAIT_LOG, /* Initiator detected error */
	0x0b, 0x4e, 0x00, 0x00, LONG_WAIT_LOG, /* Command overlap error */

	0xff, 0x40, 0x02, HARDWARE, DOWN_EQU, /* Drive hardware errors */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   ibm3584[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x83, WRONG_MODE, DOWN_EQU, /* Library not set up */
	0x02, 0x04, 0x84, DOOR_OPEN, WAIT_READY, /* I/O station is open */
	0x02, 0x04, 0x85, DOOR_OPEN, WAIT_READY, /* Door is open */
	0x02, 0xff, 0xff, 0x00, WAIT_READY,   /* becoming ready */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL, /* Destination element full */
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,  /* Source element empty */
	0x05, 0x3b, 0x80, PICKFAIL, NO_MEDIA,  /* Transport element full */
	0x05, 0x3b, 0x81, MEDIA_RESER, NO_MEDIA,  /* Element not accessible */
						/* exported by another */
						/* logical library */
	0x05, 0xff, 0xff, 0x00, ILLREQ, /* Catch-all illegal request */
	0x06, 0x28, 0xff, 0x00, IGNORE, /* Not ready-to-ready transition */
					/* I/O element accessed */
	0x06, 0x29, 0x01, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* SCSI parity error */
	0x0b, 0x48, 0x00, 0x00, LONG_WAIT_LOG, /* Initiator detected error */
	0x0b, 0x4e, 0x00, 0x00, LONG_WAIT_LOG, /* Command overlap error */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t hpslxx[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,   /* becoming ready */
	0x02, 0x04, 0x02, 0x00, WAIT_INIT,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_READY, /* Door is open */
	0x02, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x02, 0xff, 0xff, 0x00, WAIT_READY,   /* becoming ready */
	0x03, 0x30, 0x00, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x03, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x03, 0x30, 0x07, 0x00, DOWN_EQU, /* Cleaning failed */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL, /* Destination element full */
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,  /* Source element empty */
	0x05, 0x3b, 0xa0, PICKFAIL, NO_MEDIA,  /* Transport element full */
	0x05, 0xff, 0xff, 0x00, ILLREQ, /* Catch-all illegal request */
	0x06, 0x28, 0xff, 0x00, IGNORE, /* Not ready-to-ready transition */
					/* I/O element accessed */
	0x06, 0x29, 0x01, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x2A, 0x01, WRONG_MODE, DOWN_EQU, /* Mode parameters changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x0b, 0x47, 0x00, 0x00, LONG_WAIT_LOG, /* SCSI parity error */
	0x0b, 0x4e, 0x00, 0x00, LONG_WAIT_LOG, /* Command overlap error */
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t fujitsu_nm[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x01, 0x44, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x01, 0x00, WAIT_READY,
	0x02, 0x04, 0x03, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x82, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0x83, 0x00, DOWN_EQU,
	0x02, 0x04, 0x84, 0x00, DOWN_EQU,
	0x02, 0x04, 0xC2, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0xC3, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x04, 0xC4, MAN_INTER, DOWN_EQU, /* Fatal error */
	0x02, 0x30, 0x03, 0x00, LONG_WAIT_LOG, /* Cleaning cart installed */
	0x02, 0x3a, 0x02, 0x00, NO_MEDIA,
	0x02, 0x3B, 0x11, 0x00, WAIT_READY,
	0x02, 0x53, 0x02, 0x00, WAIT_READY,
	0x02, 0xff, 0xff, 0x00, WAIT_READY,
	0x04, 0x44, 0x00, HARDWARE, DOWN_EQU,
	0x04, 0x53, 0x00, HARDWARE, DOWN_EQU,
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU,
	0x05, 0x1a, 0x00, 0x00, ILLREQ,
	0x05, 0x20, 0x00, 0x00, ILLREQ,
	0x05, 0x21, 0x01, 0x00, ILLREQ,
	0x05, 0x24, 0x00, 0x00, ILLREQ,
	0x05, 0x25, 0x00, 0x00, ILLREQ,
	0x05, 0x25, 0x81, 0x00, ILLREQ,
	0x05, 0x26, 0x00, 0x00, ILLREQ,
	0x05, 0x30, 0x00, INCOMPATIBLE, INCOMPATIBLE_MEDIA,
	0x05, 0x39, 0x00, 0x00, ILLREQ,
	0x05, 0x3a, 0x00, 0x00, NO_MEDIA, /* drive not unloaded */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL,
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,
	0x05, 0x3b, 0x12, 0x00, ILLREQ,
	0x05, 0xff, 0xff, 0x00, ILLREQ,
	0x06, 0x0b, 0x02, 0x00, IGNORE,
	0x06, 0x28, 0x00, 0x00, IGNORE,
	0x06, 0x28, 0x01, 0x00, IGNORE,
	0x06, 0x28, 0x86, 0x00, WAIT_READY,
	0x06, 0x28, 0xc6, MAN_INTER, DOWN_EQU,
	0x06, 0x29, 0x00, POWER_ON, IGNORE,
	0x06, 0x2a, 0x01, 0x00, IGNORE,
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

sam_scsi_err_t   spcpyth[] = {
	0x00, 0x00, 0x00, 0x00, IGNORE,
	0x02, 0x04, 0x00, 0x00, WAIT_READY,	/* not ready */
	0x02, 0x04, 0x01, 0x00, WAIT_READY,	/* becoming ready */
	0x02, 0x04, 0x83, DOOR_OPEN, WAIT_READY, /* door is open */
	0x02, 0xff, 0xff, 0x00, WAIT_READY,   /* becoming ready */
	0x04, 0xff, 0xff, HARDWARE, DOWN_EQU, /* all hardware errors */
	0x05, 0x1a, 0x00, 0x00, ILLREQ, /* Param list length error */
	0x05, 0x20, 0x00, 0x00, ILLREQ, /* Invalid command */
	0x05, 0x21, 0x01, 0x00, ILLREQ, /* Invalid element addr */
	0x05, 0x3b, 0x0d, 0x00, D_ELE_FULL, /* Destination element full */
	0x05, 0x3b, 0x0e, 0x00, S_ELE_EMPTY,  /* Source element empty */
	0x05, 0xff, 0xff, 0x00, ILLREQ, /* Catch-all illegal request */
	0x06, 0x28, 0x00, 0x00, LOG_IGNORE, /* Inventory altered */
	0x06, 0x28, 0x01, 0x00, WAIT_READY, /* Door accessed */
	0x06, 0x28, 0xff, 0x00, IGNORE, /* Not ready-to-ready transition */
	0x06, 0x29, 0x00, POWER_ON, IGNORE,   /* power on  */
	0x06, 0x2A, 0x01, 0x00, WAIT_READY,   /* mode params changed */
	0x06, 0x3f, 0x01, 0x00, WAIT_READY,   /* microcode changed */
	0x06, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0x00, DOWN_EQU,
	0xff, 0xff, 0xff, 0xff, END_OF_LIST
};

struct codes
{
	sam_scsi_err_t *table;
	dtype_t   type;
}codes[] = {
	acl452,  DT_ACL452,
	acl2640, DT_ACL2640,
	hp_c7200, DT_HP_C7200,
	hpoplib, DT_HPLIBS,
	dlt2700, DT_DLT2700,
	metd360, DT_METD360,
	metd28,  DT_METD28,
	docstor, DT_DOCSTOR,
	exb210,  DT_EXB210,
	exbx80,  DT_EXBX80,
	speclog, DT_SPECLOG,
	stk97xx, DT_STK97XX,
	adic448, DT_ADIC448,
	adic1000, DT_ADIC1000,
	cyg1803, DT_CYGNET,
	ibm3570c, DT_3570C,
	sonydms, DT_SONYDMS,
	sonycsm, DT_SONYCSM,
	atlp3000, DT_ATLP3000,
	atl1500, DT_ATL1500,
	plasmond, DT_PLASMON_D,
	plasmong, DT_PLASMON_G,
	stklxx, DT_STKLXX,
	stksl3000, DT_SL3000,
	ibm3584, DT_IBM3584,
	adic100, DT_ADIC100,
	qual82xx, DT_QUAL82xx,
	quantumc4, DT_QUANTUMC4,
	odi_neo, DT_ODI_NEO,
	hpslxx, DT_HPSLXX,
	fujitsu_nm, DT_FJNMXX,
	spcpyth, DT_SLPYTHON,
	NULL,    0,
};

int
main()
{
	int	exit_status = 0;
	int	fd = 0;
	char	path_name[MAXPATHLEN];
	struct codes   *newcode = codes;
	sam_model_t    *model;
	sam_scsi_err_t *type;

	if (mkdir("errcodes", 0750) != 0) {
		fprintf(stderr, "Unable to mkdir errcodes: errno %d\n", errno);
		perror("mkdir:");
		exit(1);
	}

	while (newcode->table != NULL) {

		for (model = sam_model; model->short_name != NULL; model++) {
			if (model->dt == newcode->type) {
				sprintf(path_name, "errcodes/%s",
				    model->short_name);
				break;
			}
		}

		if (model->short_name == NULL) {
			printf("-----Fatal:Device not found in model table"
			    " %#x\n", newcode->type);
			exit_status = 1;
			goto next;
		} else {
			/* clear flag so we don't complain down below */
			model->dt = 0;
			if ((fd = open(path_name, O_TRUNC | O_CREAT |
			    O_WRONLY, 0666)) < 0) {
				fprintf(stderr, "Unable to create %s\n",
				    path_name);
				perror("open:");
				exit_status = 1;
				goto next;
			}
			for (type = newcode->table; type->action != END_OF_LIST;
			    type++) {
				if (write(fd, type, sizeof (sam_scsi_err_t))
				    != sizeof (sam_scsi_err_t)) {
					fprintf(stderr, "Unable to write %s\n",
					    path_name);
					perror("write:");
					exit_status = 1;
					goto next;
				}
			}

			if (write(fd, type, sizeof (sam_scsi_err_t))
			    != sizeof (sam_scsi_err_t)) {
				fprintf(stderr, "Unable to write %s\n",
				    path_name);
				perror("write:");
				exit_status = 1;
				goto next;
			}

next:
			close(fd);
			newcode++;
		}
	}

	for (model = sam_model; model->short_name != NULL; model++) {
		if ((model->dt & DT_SCSI_ROBOT_MASK) == DT_SCSI_ROBOT_MASK)
			printf("-----Warning: SCSI robot in model table has no "
			    "error codes: %#x:%s\n", model->dt,
			    model->short_name);
	}

	exit(exit_status);
}
