/*
 * sony_misc.c - misc routines for the sony
 */

/*
 *    SAM-QFS_notice_begin
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at pkg/OPENSOLARIS.LICENSE
 * or https://illumos.org/license/CDDL.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at pkg/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 *    SAM-QFS_notice_end
 */

#include <syslog.h>
#include "sony.h"

#pragma ident "$Revision: 1.15 $"


static struct st {
    int		stat_val;
    char	*stat_str;
} status_table[] = {
	PSCERR_NO_ERROR,		"PSCERR_NO_ERROR",
	PSCERR_INVALIDPARAMETER,	"PSCERR_INVALIDPARAMETER",
	PSCERR_CONNECT_ERROR,		"PSCERR_CONNECT_ERROR",
	PSCERR_TOOMANY_CONNECT,		"PSCERR_TOOMANY_CONNECT",
	PSCERR_NOT_INITIALIZED,		"PSCERR_NOT_INITIALIZED",
			/*	5 */
	PSCERR_BUSY,			"PSCERR_BUSY",
	PSCERR_SEND_ERROR,		"PSCERR_SEND_ERROR",
	PSCERR_SELECT_ERROR,		"PSCERR_SELECT_ERROR",
	PSCERR_RECV_ERROR,		"PSCERR_RECV_ERROR",
	PSCERR_DISCONNECT,		"PSCERR_DISCONNECT",
			/*	10 */
	PSCERR_NOTREADY,		"PSCERR_NOTREADY",
	PSCERR_CANT_ASSIGN,		"PSCERR_CANT_ASSIGN",
	PSCERR_NO_SUCH_BIN,		"PSCERR_NO_SUCH_BIN",
	PSCERR_NO_SUCH_CASSETTE,	"PSCERR_NO_SUCH_CASSETTE",
	PSCERR_ANOTHER_USER,		"PSCERR_ANOTHER_USER",
			/*	15 */
	PSCERR_ALREADY_RESERVED,	"PSCERR_ALREADY_RESERVED",
	PSCERR_NOT_RESERVED,		"PSCERR_NOT_RESERVED",
	PSCERR_NO_COMMUNICATION,	"PSCERR_NO_COMMUNICATION",
	PSCERR_DRIVE_IS_IN_USE,		"PSCERR_DRIVE_IS_IN_USE",
	PSCERR_BC_UNMATCH,		"PSCERR_BC_UNMATCH",
			/*	20 */
	PSCERR_BC_READ_TIMEOUT,		"PSCERR_BC_READ_TIMEOUT",
	PSCERR_TAPEIN_TIMEOUT,		"PSCERR_TAPEIN_TIMEOUT",
	PSCERR_ADPIN_TIMEOUT,		"PSCERR_ADPIN_TIMEOUT",
	PSCERR_DMS_MACRO_TIMEOUT,	"PSCERR_DMS_MACRO_TIMEOUT",
	PSCERR_DRIVE_MACRO_TIMEOUT,	"PSCERR_DRIVE_MACRO_TIMEOUT",
			/*	25	*/
	PSCERR_CLEANING_CASSETTE,	"PSCERR_CLEANING_CASSETTE",
	PSCERR_NOT_CLEANING_CASSETTE,	"PSCERR_NOT_CLEANING_CASSETTE",
	PSCERR_CLEANING_MAX_OVER,	"PSCERR_CLEANING_MAX_OVER",
	PSCERR_NO_SUCH_CLEANING,	"PSCERR_NO_SUCH_CLEANING",
	PSCERR_WAIT_MODE,		"PSCERR_WAIT_MODE",
			/*	30	*/
	PSCERR_DRIVE_NOT_USED,		"PSCERR_DRIVE_NOT_USED",
	PSCERR_BUFFER_OVERFLOW,		"PSCERR_BUFFER_OVERFLOW",
	PSCERR_WRITE_ERROR,		"PSCERR_WRITE_ERROR",
	PSCERR_CANT_MOVE,		"PSCERR_CANT_MOVE",
	PSCERR_CANT_OPEN_FILE,		"PSCERR_CANT_OPEN_FILE",
			/* 35 */
	PSCERR_PERMISSION_DENIED,	"PSCERR_PERMISSION_DENIED",
	PSCERR_NO_SUCH_JOB,		"PSCERR_NO_SUCH_JOB",
	PSCERR_UNKNOWN_CASSETTE,	"PSCERR_UNKNOWN_CASSETTE",
	PSCERR_WAIT_CANCEL,		"PSCERR_WAIT_CANCEL",
	PSCERR_ALREADY_WAIT_CLEANING,	"PSCERR_ALREADY_WAIT_CLEANING",
	PSCERR_NO_SUCH_WAIT_COMMAND,	"PSCERR_NO_SUCH_WAIT_COMMAND",
	PSCERR_ALREADY_CASSETTE,	"PSCERR_ALREADY_CASSETTE",
	PSCERR_CASSETTE_IS_BUSY,	"PSCERR_CASSETTE_IS_BUSY",
	PSCERR_REG_NO_MORE_ITEMS,	"PSCERR_REG_NO_MORE_ITEMS",
	PSCERR_REG_ERROR,		"PSCERR_REG_ERROR",
	PSCERR_NOT_WAIT_MODE,		"PSCERR_NOT_WAIT_MODE",
	PSCERR_NO_ADMITTANCE_LOGIN,	"PSCERR_NO_ADMITTANCE_LOGIN",
	PSCERR_INCORRECT_PASSWORD,	"PSCERR_INCORRECT_PASSWORD",
	PSCERR_CLEANING_TIMEOUT,	"PSCERR_CLEANING_TIMEOUT",
	PSCERR_CLEANING_ERROR,		"PSCERR_CLEANING_ERROR",
	PSCERR_ADPOUT_TIMEOUT,		"PSCERR_ADPOUT_TIMEOUT",
	PSCERR_CASSETTE_UNMATCH,	"PSCERR_CASSETTE_UNMATCH",
	PSCERR_LOADED_TIMEOUT,		"PSCERR_LOADED_TIMEOUT",
	PSCERR_SYSTEM_CALL_ERROR,	"PSCERR_SYSTEM_CALL_ERROR",
	PSCERR_MMT_REMOTE_MODE,		"PSCERR_MMT_REMOTE_MODE",
	PSCERR_MMT_TEST_MODE,		"PSCERR_MMT_TEST_MODE",
	PSCERR_WIN_SEARCH_ERROR,	"PSCERR_WIN_SEARCH_ERROR",
	PSCERR_INITIALIZE_ERROR,	"PSCERR_INITIALIZE_ERROR",
	PSCERR_BC_READ_ERROR,		"PSCERR_BC_READ_ERROR",
	PSCERR_COMMAND_ERROR2,		"PSCERR_COMMAND_ERROR2",
	PSCERR_DISTURBED,		"PSCERR_DISTURBED",
	PSCERR_ADAPTOR_EMG_MODE2,	"PSCERR_ADAPTOR_EMG_MODE2",
	PSCERR_IO_PORT_BUSY2,		"PSCERR_IO_PORT_BUSY2",
	PSCERR_INITIALIZING2,		"PSCERR_INITIALIZING2",
	PSCERR_SERVO_OFF2,		"PSCERR_SERVO_OFF2",
	PSCERR_CASS_SIZE_ERRR2,		"PSCERR_CASS_SIZE_ERRR2",
	PSCERR_CASS_EXIST_JCT2,		"PSCERR_CASS_EXIST_JCT2",
	PSCERR_NO_COMM_CC2,		"PSCERR_NO_COMM_CC2",
	PSCERR_WINDOW_NOT_INIT2,	"PSCERR_WINDOW_NOT_INIT2",
	PSCERR_BIN_NOT_INIT2,		"PSCERR_BIN_NOT_INIT2",
	PSCERR_DRIVE_PULLED2,		"PSCERR_DRIVE_PULLED2",
	PSCERR_OUTPORT_FULL2,		"PSCERR_OUTPORT_FULL2",
	PSCERR_TEST_MODE2,		"PSCERR_TEST_MODE2",
	PSCERR_DOOR_OPEN2,		"PSCERR_DOOR_OPEN2",
	PSCERR_BIN_TYPE_ERROR2,		"PSCERR_BIN_TYPE_ERROR2",
	PSCERR_NO_BIN2,			"PSCERR_NO_BIN2",
	PSCERR_CASS_EXIST_ELV2,		"PSCERR_CASS_EXIST_ELV2",
	PSCERR_CARRIER_PUT_TROUBLE2,	"PSCERR_CARRIER_PUT_TROUBLE2",
	PSCERR_CARRIER_GET_TROUBLE2,	"PSCERR_CARRIER_GET_TROUBLE2",
	PSCERR_DEST_CASS_EXIST2,	"PSCERR_DEST_CASS_EXIST2",
	PSCERR_SOURCE_NO_CASS2,		"PSCERR_SOURCE_NO_CASS2",
	PSCERR_COMMAND_ERROR1,		"PSCERR_COMMAND_ERROR1",
	PSCERR_ADAPTOR_EMG_MODE1,	"PSCERR_ADAPTOR_EMG_MODE1",
	PSCERR_IO_PORT_BUSY1,		"PSCERR_IO_PORT_BUSY1",
	PSCERR_INITIALIZING1,		"PSCERR_INITIALIZING1",
	PSCERR_SERVO_OFF1,		"PSCERR_SERVO_OFF1",
	PSCERR_CASS_SIZE_ERRR1,		"PSCERR_CASS_SIZE_ERRR1",
	PSCERR_CASS_EXIST_JCT1,		"PSCERR_CASS_EXIST_JCT1",
	PSCERR_NO_COMM_CC1,		"PSCERR_NO_COMM_CC1",
	PSCERR_ELV_TEST_MODE1,		"PSCERR_ELV_TEST_MODE1",
	PSCERR_WINDOW_NOT_INIT1, 	"PSCERR_WINDOW_NOT_INIT1",
	PSCERR_BIN_NOT_INIT1,		"PSCERR_BIN_NOT_INIT1",
	PSCERR_DRIVE_PULLED1,		"PSCERR_DRIVE_PULLED1",
	PSCERR_OUTPORT_FULL1,		"PSCERR_OUTPORT_FULL1",
	PSCERR_TEST_MODE1,		"PSCERR_TEST_MODE1",
	PSCERR_DOOR_OPEN1,		"PSCERR_DOOR_OPEN1",
	PSCERR_BIN_TYPE_ERROR1,		"PSCERR_BIN_TYPE_ERROR1",
	PSCERR_NO_BIN1,			"PSCERR_NO_BIN1",
	PSCERR_CASS_EXIST_ELV1,		"PSCERR_CASS_EXIST_ELV1",
	PSCERR_DEST_CASS_EXIST1,	"PSCERR_DEST_CASS_EXIST1",
	PSCERR_SOURCE_NO_CASS1,		"PSCERR_SOURCE_NO_CASS1",
	PSCERR_ELV_NOT_CONNECT2,	"PSCERR_ELV_NOT_CONNECT2",
	PSCERR_SERVO_NO_COMM2,		"PSCERR_SERVO_NO_COMM2",
	PSCERR_ADAPTOR_NO_COMM2,	"PSCERR_ADAPTOR_NO_COMM2",
	PSCERR_JUNCTION_NO_COMM2,	"PSCERR_JUNCTION_NO_COMM2",
	PSCERR_IO_NO_COMM2,		"PSCERR_IO_NO_COMM2",
	PSCERR_POWER_FAILURE2,		"PSCERR_POWER_FAILURE2",
	PSCERR_CARRIER_GET_POSITION2,	"PSCERR_CARRIER_GET_POSITION2",
	PSCERR_CARRIER_PUT_POSITION2,	"PSCERR_CARRIER_PUT_POSITION2",
	PSCERR_IF_TROUBLE2,		"PSCERR_IF_TROUBLE2",
	PSCERR_BC_READER_TRBL2,		"PSCERR_BC_READER_TRBL2",
	PSCERR_ELEVATOR_TROUBLE2,	"PSCERR_ELEVATOR_TROUBLE2",
	PSCERR_SERVO_TROUBLE2,		"PSCERR_SERVO_TROUBLE2",
	PSCERR_DEST_TROUBLE2,		"PSCERR_DEST_TROUBLE2",
	PSCERR_SOURCE_TROUBLE2,		"PSCERR_SOURCE_TROUBLE2",
	PSCERR_ADAPTOR_TROUBLE2,	"PSCERR_ADAPTOR_TROUBLE2",
	PSCERR_JUNCTION_TROUBLE2,	"PSCERR_JUNCTION_TROUBLE2",
	PSCERR_IO_PORT_TROUBLE2,	"PSCERR_IO_PORT_TROUBLE",
	PSCERR_BIN_TROUBLE2,		"PSCERR_BIN_TROUBLE2",
	PSCERR_ELV_NOT_CONNECT1,	"PSCERR_ELV_NOT_CONNECT1",
	PSCERR_SERVO_NO_COMM1,		"PSCERR_SERVO_NO_COMM1",
	PSCERR_ADAPTOR_NO_COMM1,	"PSCERR_ADAPTOR_NO_COMM1",
	PSCERR_JUNCTION_NO_COMM1,	"PSCERR_JUNCTION_NO_COMM1",
	PSCERR_IO_NO_COMM1,		"PSCERR_IO_NO_COMM1",
	PSCERR_POWER_FAILURE1,		"PSCERR_POWER_FAILURE1",
	PSCERR_CARRIER_GET_POSITION1,	"PSCERR_CARRIER_GET_POSITION1",
	PSCERR_CARRIER_PUT_POSITION1,	"PSCERR_CARRIER_PUT_POSITION1",
	PSCERR_BC_READER_TRBL1,		"PSCERR_BC_READER_TRBL1",
	PSCERR_ELEVATOR_TROUBLE1,	"PSCERR_ELEVATOR_TROUBLE1",
	PSCERR_SERVO_TROUBLE1,		"PSCERR_SERVO_TROUBLE1",
	PSCERR_DEST_TROUBLE1,		"PSCERR_DEST_TROUBLE1",
	PSCERR_SOURCE_TROUBLE1,		"PSCERR_SOURCE_TROUBLE1",
	PSCERR_ADAPTOR_TROUBLE1,	"PSCERR_ADAPTOR_TROUBLE1",
	PSCERR_JUNCTION_TROUBLE1,	"PSCERR_JUNCTION_TROUBLE1",
	PSCERR_IO_PORT_TROUBLE1,	"PSCERR_IO_PORT_TROUBLE1",
	PSCERR_BIN_TROUBLE1,		"PSCERR_BIN_TROUBLE1",
	PSCERR_NUMBER_IN_USE,		"PSCERR_NUMBER_IN_USE",
	PSCERR_ABNOMAL_END,		"PSCERR_ABNOMAL_END",
	PSCERR_IMPOSSIBLE_EXEC,		"PSCERR_IMPOSSIBLE_EXEC",
	PSCERR_MACRO_BUSY,		"PSCERR_MACRO_BUSY",
	PSCERR_NO_SUCH_NUMBER,		"PSCERR_NO_SUCH_NUMBER",
	PSCERR_MACRO_CANCEL1,		"PSCERR_MACRO_CANCEL1",
	PSCERR_MACRO_CANCEL2,		"PSCERR_MACRO_CANCEL2",
	PSCERR_NOT_READY,		"PSCERR_NOT_READY",
	PSCERR_LOADING,			"PSCERR_LOADING",
	PSCERR_NOT_REMOTE_MODE,		"PSCERR_NOT_REMOTE_MODE",
	PSCERR_CLEANING_MODE,		"PSCERR_CLEANING_MODE",
	PSCERR_MEDIUM_NOT_PRESENT,	"PSCERR_MEDIUM_NOT_PRESENT",
	PSCERR_DRV_TAPE_EXIST,		"PSCERR_DRV_TAPE_EXIST",
	PSCERR_NOT_INJECT,		"PSCERR_NOT_INJECT",
	PSCERR_ERR_LOAD_FAIL,		"PSCERR_ERR_LOAD_FAIL",
	PSCERR_RESET,			"PSCERR_RESET",
	PSCERR_EJECTING,		"PSCERR_EJECTING",
	PSCERR_MEDIUM_ERROR,		"PSCERR_MEDIUM_ERROR",
	PSCERR_SERVO_ERROR,		"PSCERR_SERVO_ERROR",
	PSCERR_READ_FAIL,		"PSCERR_READ_FAIL",
	PSCERR_INCOMPATIBLE_FORMAT,	"PSCERR_INCOMPATIBLE_FORMAT",
	PSCERR_FORMAT_CORRUPTED,	"PSCERR_FORMAT_CORRUPTED",
	PSCERR_ID_SEARCH_ERROR,		"PSCERR_ID_SEARCH_ERROR",
	PSCERR_REACH_BOT,		"PSCERR_REACH_BOT",
	PSCERR_REACH_EOT,		"PSCERR_REACH_EOT",
	PSCERR_WRITE_FAIL,		"PSCERR_WRITE_FAIL",
	PSCERR_SYNC_FAIL,		"PSCERR_SYNC_FAIL",
	PSCERR_LOST_LOCK_ERROR,		"PSCERR_LOST_LOCK_ERROR",
	PSCERR_UNCORRECTED_ERROR,	"PSCERR_UNCORRECTED_ERROR",
	PSCERR_DATA_RATE,		"PSCERR_DATA_RATE",
	PSCERR_DRIVE_PARITY_ERROR,	"PSCERR_DRIVE_PARITY_ERROR",
	PSCERR_RECOVER_REQUEST,		"PSCERR_RECOVER_REQUEST",
	PSCERR_RECOVER_FAILED,		"PSCERR_RECOVER_FAILED",
	PSCERR_UNFORMATTED_TAPE,	"PSCERR_UNFORMATTED_TAPE",
	PSCERR_UPDATE_FLAG_ON,		"PSCERR_UPDATE_FLAG_ON",
	PSCERR_HEAD_BREAK,		"PSCERR_HEAD_BREAK",
	PSCERR_HARDWARE_ERROR,		"PSCERR_HARDWARE_ERROR",
	PSCERR_MESSAGE_ERROR,		"PSCERR_MESSAGE_ERROR",
	PSCERR_SCSI_PARITY_ERROR,	"PSCERR_SCSI_PARITY_ERROR",
	PSCERR_DRIVE_TIMEOUT,		"PSCERR_DRIVE_TIMEOUT",
	PSCERR_DRV_INVALID_COMMAND,	"PSCERR_DRV_INVALID_COMMAND",
	PSCERR_DRV_INVALID_PARAMETER,	"PSCERR_DRV_INVALID_PARAMETER",
	PSCERR_DRV_CMD_SEQ_ERROR,	"PSCERR_DRV_CMD_SEQ_ERROR",
	PSCERR_INVALID_TAPE_POINTER,	"PSCERR_INVALID_TAPE_POINTER",
	PSCERR_WRITE_PROTECTED,		"PSCERR_WRITE_PROTECTED",
	PSCERR_DRIVE_NOT_CONNECTED,	"PSCERR_DRIVE_NOT_CONNECTED",
	PSCERR_DRIVE_NOT_LOADED,	"PSCERR_DRIVE_NOT_LOADED",
	PSCERR_INDEFINITENESS_DATA,	"PSCERR_INDEFINITENESS_DATA",
	PSCERR_DRIVE_SYSTEM_ERROR,	"PSCERR_DRIVE_SYSTEM_ERROR",
	PSCERR_DRIVE_BUSY,		"PSCERR_DRIVE_BUSY",
	PSCERR_DRIVE_NOT_SUPPORTED,	"PSCERR_DRIVE_NOT_SUPPORTED",
	PSCERR_DRIVE_OTHER_ERROR,	"PSCERR_DRIVE_OTHER_ERROR",
	PSCERR_SAMFS_LAST,	"code %d",
};

static  char unknown[32]; /* unknown buffer type */

/*
 *      Procedure Type Declarations:
 */
char *
sam_sony_status(int status)
{
	int	i;

	for (i = 0; status_table[i].stat_val != PSCERR_SAMFS_LAST; i++) {
		if (status == status_table[i].stat_val)
			return (status_table[i].stat_str);
	}
	sprintf(unknown, status_table[i].stat_str, status);
	return (unknown);
}


/*
 * is_storage - is the specified element a storage address
 */
int
is_storage(library_t *library, const uint_t element)
{
	return (TRUE);
}


/*
 * slot_number - given the element address, return the slot number
 */
uint_t
slot_number(library_t *library, const uint_t element)
{
	return (element);
}


/*
 * element_address - given the slot number, return the element address
 */
uint_t
element_address(library_t *library, const uint_t slot)
{
	return (slot);
}
