/*
 * stk_misc.c - misc routines for the stk
 */

/*
 *    SAM-QFS_notice_begin
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at pkg/OPENSOLARIS.LICENSE
 * or https://illumos.org/license/CDDL.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at pkg/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 *    SAM-QFS_notice_end
 */

#include <stdio.h>
#include <syslog.h>
#include "stk.h"

#include "sam/lib.h"

#pragma ident "$Revision: 1.20 $"

/*
 * The following was pulled from the stk release common_lib/cl_status.c
 * To get the status printed required linking all the libraries which
 * also required more system libraries.
 *
 *
 * NOTE: When adding to this table, please add elements in the same
 * order they are found in db_defs.h.  It's not required for the
 * software to work, but it provides easier cross-reference
 */
static struct st {
    STATUS    stat_val;
    char    *stat_str;
} status_table[] = {
    STATUS_SUCCESS,    "STATUS_SUCCESS",
    STATUS_ACS_FULL,    "STATUS_ACS_FULL",
    STATUS_ACS_NOT_IN_LIBRARY,    "STATUS_ACS_NOT_IN_LIBRARY",
    STATUS_ACS_OFFLINE,    "STATUS_ACS_OFFLINE",
    STATUS_ACSLM_IDLE,    "STATUS_ACSLM_IDLE",
					/*   5 */
    STATUS_ACTIVITY_END,    "STATUS_ACTIVITY_END",
    STATUS_ACTIVITY_START,    "STATUS_ACTIVITY_START",
    STATUS_AUDIT_ACTIVITY,    "STATUS_AUDIT_ACTIVITY",
    STATUS_AUDIT_IN_PROGRESS,    "STATUS_AUDIT_IN_PROGRESS",
    STATUS_CANCELLED,    "STATUS_CANCELLED",
					/*  10 */
    STATUS_CAP_AVAILABLE,    "STATUS_CAP_AVAILABLE",
    STATUS_CAP_FULL,    "STATUS_CAP_FULL",
    STATUS_CAP_IN_USE,    "STATUS_CAP_IN_USE",
    STATUS_CELL_EMPTY,    "STATUS_CELL_EMPTY",
    STATUS_CELL_FULL,    "STATUS_CELL_FULL",
					/*  15 */
    STATUS_CELL_INACCESSIBLE,    "STATUS_CELL_INACCESSIBLE",
    STATUS_CELL_RESERVED,    "STATUS_CELL_RESERVED",
    STATUS_CLEAN_DRIVE,    "STATUS_CLEAN_DRIVE",
    STATUS_COMMUNICATION_FAILED,    "STATUS_COMMUNICATION_FAILED",
    STATUS_CONFIGURATION_ERROR,    "STATUS_CONFIGURATION_ERROR",
					/*  20 */
    STATUS_COUNT_TOO_SMALL,    "STATUS_COUNT_TOO_SMALL",
    STATUS_COUNT_TOO_LARGE,    "STATUS_COUNT_TOO_LARGE",
    STATUS_CURRENT,    "STATUS_CURRENT",
    STATUS_DATABASE_ERROR,    "STATUS_DATABASE_ERROR",
    STATUS_DEGRADED_MODE,    "STATUS_DEGRADED_MODE",
					/*  25 */
    STATUS_DONE,    "STATUS_DONE",
    STATUS_DOOR_CLOSED,    "STATUS_DOOR_CLOSED",
    STATUS_DOOR_OPENED,    "STATUS_DOOR_OPENED",
    STATUS_DRIVE_AVAILABLE,    "STATUS_DRIVE_AVAILABLE",
    STATUS_DRIVE_IN_USE,    "STATUS_DRIVE_IN_USE",
					/*  30 */
    STATUS_DRIVE_NOT_IN_LIBRARY,    "STATUS_DRIVE_NOT_IN_LIBRARY",
    STATUS_DRIVE_OFFLINE,    "STATUS_DRIVE_OFFLINE",
    STATUS_DRIVE_RESERVED,    "STATUS_DRIVE_RESERVED",
    STATUS_DUPLICATE_LABEL,    "STATUS_DUPLICATE_LABEL",
    STATUS_EJECT_ACTIVITY,    "STATUS_EJECT_ACTIVITY",
					/*  35 */
    STATUS_ENTER_ACTIVITY,    "STATUS_ENTER_ACTIVITY",
    STATUS_EVENT_LOG_FULL,    "STATUS_EVENT_LOG_FULL",
    STATUS_IDLE_PENDING,    "STATUS_IDLE_PENDING",
    STATUS_INPUT_CARTRIDGES,    "STATUS_INPUT_CARTRIDGES",
    STATUS_INVALID_ACS,    "STATUS_INVALID_ACS",
					/*  40 */
    STATUS_INVALID_COLUMN,    "STATUS_INVALID_COLUMN",
    STATUS_INVALID_COMMAND,    "STATUS_INVALID_COMMAND",
    STATUS_INVALID_DRIVE,    "STATUS_INVALID_DRIVE",
    STATUS_INVALID_LSM,    "STATUS_INVALID_LSM",
    STATUS_INVALID_MESSAGE,    "STATUS_INVALID_MESSAGE",
					/*  45 */
    STATUS_INVALID_OPTION,    "STATUS_INVALID_OPTION",
    STATUS_INVALID_PANEL,    "STATUS_INVALID_PANEL",
    STATUS_INVALID_PORT,    "STATUS_INVALID_PORT",
    STATUS_INVALID_ROW,    "STATUS_INVALID_ROW",
    STATUS_INVALID_STATE,    "STATUS_INVALID_STATE",
					/*  50 */
    STATUS_INVALID_SUBPANEL,    "STATUS_INVALID_SUBPANEL",
    STATUS_INVALID_TYPE,    "STATUS_INVALID_TYPE",
    STATUS_INVALID_VALUE,    "STATUS_INVALID_VALUE",
    STATUS_INVALID_VOLUME,    "STATUS_INVALID_VOLUME",
    STATUS_IPC_FAILURE,    "STATUS_IPC_FAILURE",
					/*  55 */
    STATUS_LIBRARY_BUSY,    "STATUS_LIBRARY_BUSY",
    STATUS_LIBRARY_FAILURE,    "STATUS_LIBRARY_FAILURE",
    STATUS_LIBRARY_NOT_AVAILABLE,   "STATUS_LIBRARY_NOT_AVAILABLE",
    STATUS_LOCATION_OCCUPIED,    "STATUS_LOCATION_OCCUPIED",
    STATUS_LSM_FULL,    "STATUS_LSM_FULL",
					/*  60 */
    STATUS_LSM_NOT_IN_LIBRARY,    "STATUS_LSM_NOT_IN_LIBRARY",
    STATUS_LSM_OFFLINE,    "STATUS_LSM_OFFLINE",
    STATUS_MESSAGE_NOT_FOUND,    "STATUS_MESSAGE_NOT_FOUND",
    STATUS_MESSAGE_TOO_LARGE,    "STATUS_MESSAGE_TOO_LARGE",
    STATUS_MESSAGE_TOO_SMALL,    "STATUS_MESSAGE_TOO_SMALL",
					/*  65 */
    STATUS_MISPLACED_TAPE,    "STATUS_MISPLACED_TAPE",
    STATUS_MULTI_ACS_AUDIT,    "STATUS_MULTI_ACS_AUDIT",
    STATUS_NORMAL,    "STATUS_NORMAL",
    STATUS_NONE,    "STATUS_NONE",
    STATUS_NOT_IN_SAME_ACS,    "STATUS_NOT_IN_SAME_ACS",
					/*  70 */
    STATUS_ONLINE,    "STATUS_ONLINE",
    STATUS_OFFLINE,    "STATUS_OFFLINE",
    STATUS_PENDING,    "STATUS_PENDING",
    STATUS_PORT_NOT_IN_LIBRARY,    "STATUS_PORT_NOT_IN_LIBRARY",
    STATUS_PROCESS_FAILURE,    "STATUS_PROCESS_FAILURE",
					/*  75 */
    STATUS_RECOVERY_COMPLETE,    "STATUS_RECOVERY_COMPLETE",
    STATUS_RECOVERY_FAILED,    "STATUS_RECOVERY_FAILED",
    STATUS_RECOVERY_INCOMPLETE,    "STATUS_RECOVERY_INCOMPLETE",
    STATUS_RECOVERY_STARTED,    "STATUS_RECOVERY_STARTED",
    STATUS_REMOVE_CARTRIDGES,    "STATUS_REMOVE_CARTRIDGES",
					/*  80 */
    STATUS_RETRY,    "STATUS_RETRY",
    STATUS_STATE_UNCHANGED,    "STATUS_STATE_UNCHANGED",
    STATUS_TERMINATED,    "STATUS_TERMINATED",
    STATUS_VALID,    "STATUS_VALID",
    STATUS_VALUE_UNCHANGED,    "STATUS_VALUE_UNCHANGED",
					/*  85 */
    STATUS_VARY_DISALLOWED,    "STATUS_VARY_DISALLOWED",
    STATUS_VOLUME_ADDED,    "STATUS_VOLUME_ADDED",
    STATUS_VOLUME_EJECTED,    "STATUS_VOLUME_EJECTED",
    STATUS_VOLUME_ENTERED,    "STATUS_VOLUME_ENTERED",
    STATUS_VOLUME_FOUND,    "STATUS_VOLUME_FOUND",
					/*  90 */
    STATUS_VOLUME_HOME,    "STATUS_VOLUME_HOME",
    STATUS_VOLUME_IN_DRIVE,    "STATUS_VOLUME_IN_DRIVE",
    STATUS_VOLUME_IN_TRANSIT,    "STATUS_VOLUME_IN_TRANSIT",
    STATUS_VOLUME_NOT_IN_DRIVE,    "STATUS_VOLUME_NOT_IN_DRIVE",
    STATUS_VOLUME_NOT_IN_LIBRARY,   "STATUS_VOLUME_NOT_IN_LIBRARY",
					/*  95 */
    STATUS_UNREADABLE_LABEL,    "STATUS_UNREADABLE_LABEL",
    STATUS_UNSUPPORTED_OPTION,    "STATUS_UNSUPPORTED_OPTION",
    STATUS_UNSUPPORTED_STATE,    "STATUS_UNSUPPORTED_STATE",
    STATUS_UNSUPPORTED_TYPE,    "STATUS_UNSUPPORTED_TYPE",
    STATUS_VOLUME_IN_USE,    "STATUS_VOLUME_IN_USE",
					/* 100 */
    STATUS_PORT_FAILURE,    "STATUS_PORT_FAILURE",
    STATUS_MAX_PORTS,    "STATUS_MAX_PORTS",
    STATUS_PORT_ALREADY_OPEN,    "STATUS_PORT_ALREADY_OPEN",
    STATUS_QUEUE_FAILURE,    "STATUS_QUEUE_FAILURE",
    STATUS_NI_FAILURE,    "STATUS_NI_FAILURE",
					/* 105 */
    STATUS_NI_TIMEDOUT,    "STATUS_NI_TIMEDOUT",
    STATUS_INVALID_COMM_SERVICE,    "STATUS_INVALID_COMM_SERVICE",
    STATUS_COMPLETE,    "STATUS_COMPLETE",
    STATUS_AUDIT_FAILED,    "STATUS_AUDIT_FAILED",
    STATUS_NO_PORTS_ONLINE,    "STATUS_NO_PORTS_ONLINE",
					/* 110 */
    STATUS_CARTRIDGES_IN_CAP,    "STATUS_CARTRIDGES_IN_CAP",
    STATUS_TRANSLATION_FAILURE,    "STATUS_TRANSLATION_FAILURE",
    STATUS_DATABASE_DEADLOCK,    "STATUS_DATABASE_DEADLOCK",
    STATUS_DIAGNOSTIC,    "STATUS_DIAGNOSTIC",
    STATUS_DUPLICATE_IDENTIFIER,    "STATUS_DUPLICATE_IDENTIFIER",
					/* 115 */
    STATUS_EVENT_LOG_FAILURE,    "STATUS_EVENT_LOG_FAILURE",
    STATUS_DISMOUNT_ACTIVITY,    "STATUS_DISMOUNT_ACTIVITY",
    STATUS_MOUNT_ACTIVITY,    "STATUS_MOUNT_ACTIVITY",
    STATUS_POOL_NOT_FOUND,    "STATUS_POOL_NOT_FOUND",
    STATUS_POOL_NOT_EMPTY,    "STATUS_POOL_NOT_EMPTY",
					/* 120 */
    STATUS_INVALID_RANGE,    "STATUS_INVALID_RANGE",
    STATUS_INVALID_POOL,    "STATUS_INVALID_POOL",
    STATUS_POOL_HIGH_WATER,    "STATUS_POOL_HIGH_WATER",
    STATUS_POOL_LOW_WATER,    "STATUS_POOL_LOW_WATER",
    STATUS_INVALID_VERSION,    "STATUS_INVALID_VERSION",
					/* 125 */
    STATUS_MISSING_OPTION,    "STATUS_MISSING_OPTION",
    STATUS_INCORRECT_ATTRIBUTE,    "STATUS_INCORRECT_ATTRIBUTE",
    STATUS_INVALID_LOCKID,    "STATUS_INVALID_LOCKID",
    STATUS_VOLUME_AVAILABLE,    "STATUS_VOLUME_AVAILABLE",
    STATUS_READABLE_LABEL,    "STATUS_READABLE_LABEL",
					/* 130 */
    STATUS_NO_CAP_AVAILABLE,    "STATUS_NO_CAP_AVAILABLE",
    STATUS_LOCK_FAILED,    "STATUS_LOCK_FAILED",
    STATUS_DEADLOCK,    "STATUS_DEADLOCK",
    STATUS_LOCKID_NOT_FOUND,    "STATUS_LOCKID_NOT_FOUND",
    STATUS_INCORRECT_LOCKID,    "STATUS_INCORRECT_LOCKID",
					/* 135 */
    STATUS_SCRATCH_NOT_AVAILABLE,   "STATUS_SCRATCH_NOT_AVAILABLE",
    STATUS_CLEAN_DRIVE_COMPLETE,    "STATUS_CLEAN_DRIVE_COMPLETE",
    STATUS_VOLUME_NOT_FOUND,    "STATUS_VOLUME_NOT_FOUND",
    STATUS_CAP_DOOR_OPEN,    "STATUS_CAP_DOOR_OPEN",
    STATUS_CAP_INOPERATIVE,    "STATUS_CAP_INOPERATIVE",
					/* 140 */
    STATUS_DISK_FULL,    "STATUS_DISK_FULL",
    STATUS_CAP_NOT_IN_LIBRARY,    "STATUS_CAP_NOT_IN_LIBRARY",
    STATUS_CAP_OFFLINE,    "STATUS_CAP_OFFLINE",
    STATUS_INVALID_CAP,    "STATUS_INVALID_CAP",
    STATUS_INCORRECT_CAP_MODE,    "STATUS_INCORRECT_CAP_MODE",
					/* 145 */
    STATUS_INCORRECT_STATE,    "STATUS_INCORRECT_STATE",
    STATUS_VARY_IN_PROGRESS,    "STATUS_VARY_IN_PROGRESS",
    STATUS_ACS_ONLINE,    "STATUS_ACS_ONLINE",
    STATUS_AUTOMATIC,    "STATUS_AUTOMATIC",
    STATUS_MANUAL,    "STATUS_MANUAL",
					/* 150 */
    STATUS_VOLUME_DELETED,    "STATUS_VOLUME_DELETED",
    STATUS_INSERT_MAGAZINES,    "STATUS_INSERT_MAGAZINES",
    STATUS_UNSUPPORTED_COMMAND,    "STATUS_UNSUPPORTED_COMMAND",
    STATUS_COMMAND_ACCESS_DENIED,   "STATUS_COMMAND_ACCESS_DENIED",
    STATUS_VOLUME_ACCESS_DENIED,    "STATUS_VOLUME_ACCESS_DENIED",
					/* 155 */
    STATUS_OWNER_NOT_FOUND,    "STATUS_OWNER_NOT_FOUND",
    STATUS_INVALID_DRIVE_TYPE,    "STATUS_INVALID_DRIVE_TYPE",
    STATUS_INVALID_MEDIA_TYPE,    "STATUS_INVALID_MEDIA_TYPE",
    STATUS_INCOMPATIBLE_MEDIA_TYPE, "STATUS_INCOMPATIBLE_MEDIA_TYPE",
    STATUS_DRIVE_FOUND,		    "STATUS_DRIVE_FOUND",
				    /* 160 */
    STATUS_CAP_DONE,		    "STATUS_CAP_DONE",
    STATUS_INVALID_SUBSYSID,	    "STATUS_INVALID_SUBSYSID",
    STATUS_LSM_OFFLINE_PENDING,	    "STATUS_LSM_OFFLINE_PENDING",
    STATUS_PORT_OFFLINE,	    "STATUS_PORT_OFFLINE",
    STATUS_COMMUNICATING,			"STATUS_COMMUNICATING",
				/* 165 */
    STATUS_NOT_COMMUNICATING,		"STATUS_NOT_COMMUNICATING",
    STATUS_IDLE,					"STATUS_IDLE",
    STATUS_OFFLINE_PENDING,			"STATUS_OFFLINE_PENDING",
    STATUS_SWITCHOVER_INITIATED,	"STATUS_SWITCHOVER_INITIATED",
    STATUS_SWITCHOVER_RECOVERY_COMPLETE, "STATUS_SWITCHOVER_RECOVERY_COMPLETE",
				/* 170 */
    STATUS_LMU_STATUS_CHANGE_NEW_MASTER,
				"STATUS_LMU_STATUS_CHANGE_NEW_MASTER",
    STATUS_LMU_STATUS_CHANGE_STANDBY_COMM,
				"STATUS_LMU_STATUS_CHANGE_STANDBY_COMM",
    STATUS_LMU_STATUS_CHANGE_STANDBY_NOT_COMM,
				"STATUS_LMU_STATUS_CHANGE_STANDBY_NOT_COMM",
    STATUS_STANDBY_LMU_NO_PORTS,	"STATUS_STANDBY_LMU_NO_PORTS",
    STATUS_SWITCHOVER_IN_PROGRESS,	"STATUS_SWITCHOVER_IN_PROGRESS",
				/* 175 */
    STATUS_LAST_MASTER_PORT,		"STATUS_LAST_MASTER_PORT",
    STATUS_NOT_CONFIGD_DUAL,		"STATUS_NOT_CONFIGD_DUAL",
    STATUS_DISALLOWED_ON_MASTER,	"STATUS_DISALLOWED_ON_MASTER",
    STATUS_LMU_TO_LMU_LINK_BAD,		"STATUS_LMU_TO_LMU_LINK_BAD",
    STATUS_TOO_MANY_NAKS,			"STATUS_TOO_MANY_NAKS",
				/* 180 */
    STATUS_MASTER_NOT_RESPONDING,	"STATUS_MASTER_NOT_RESPONDING",
    STATUS_MOVE_ACTIVITY,			"STATUS_MOVE_ACTIVITY",
    STATUS_VOLUME_MISSING,			"STATUS_VOLUME_MISSING",
    STATUS_VOLUME_BEING_RECOVERED, 	"STATUS_VOLUME_BEING_RECOVERED",
    STATUS_INVALID_CLIENT,			"STATUS_INVALID_CLIENT",
					/* 185 */
    STATUS_VOLUME_REACTIVATED,		"STATUS_VOLUME_REACTIVATED",
    STATUS_VOLUME_ABSENT,			"STATUS_VOLUME_ABSENT",
    STATUS_UNIT_ATTENTION,			"STATUS_UNIT_ATTENTION",
    STATUS_READY,					"STATUS_READY",
    STATUS_NOT_READY,				"STATUS_NOT_READY",
				    /* 190 */
    STATUS_SERIAL_NUM_CHG,			"STATUS_SERIAL_NUM_CHG",
    STATUS_HARDWARE_ERROR,			"STATUS_HARDWARE_ERROR",
    STATUS_MONITOR_COMPLETE,		"STATUS_MONITOR_COMPLETE",
    STATUS_PTP_NOT_FOUND,			"STATUS_PTP_NOT_FOUND",
    STATUS_INVALID_EVENT_CLASS,		"STATUS_INVALID_EVENT_CLASS",
				    /* 195 */
    STATUS_HAND_NOT_FOUND,			"STATUS_HAND_NOT_FOUND",
    STATUS_LMU_NOT_FOUND,			"STATUS_LMU_NOT_FOUND",
    STATUS_OPERATIVE,				"STATUS_OPERATIVE",
    STATUS_INOPERATIVE,				"STATUS_INOPERATIVE",
    STATUS_MAINT_REQUIRED,			"STATUS_MAINT_REQUIRED",
				    /* 200 */
    STATUS_CAP_DOOR_CLOSED,			"STATUS_CAP_DOOR_CLOSED",
    STATUS_LSM_TYPE_CHG,			"STATUS_LSM_TYPE_CHG",
    STATUS_CONFIGURATION_CHANGED,	"STATUS_CONFIGURATION_CHANGED",
    STATUS_DRIVE_TYPE_CHG,			"STATUS_DRIVE_TYPE_CHG",
    STATUS_NI_TIMEOUT,				"STATUS_NI_TIMEOUT",
				    /* 205 */
    STATUS_VOLUME_OVER_MAX_CLEAN,	"STATUS_VOLUME_OVER_MAX_CLEAN",
    STATUS_VOLUME_CLEAN_CART_SPENT,	"STATUS_VOLUME_CLEAN_CART_SPENT",
    STATUS_DRIVE_ADDED,				"STATUS_DRIVE_ADDED",
    STATUS_DRIVE_REMOVED,			"STATUS_DRIVE_REMOVED",
    STATUS_LMU_TYPE_CHG,			"STATUS_LMU_TYPE_CHG",
				    /* 210 */

    STATUS_LMU_COMPAT_LVL_CHG,		"STATUS_LMU_COMPAT_LVL_CHG",
    STATUS_LMU_NOW_STANDALONE,		"STATUS_LMU_NOW_STANDALONE",
    STATUS_LMU_NOW_MASTER,			"STATUS_LMU_NOW_MASTER",
    STATUS_LMU_NOW_STANDBY,			"STATUS_LMU_NOW_STANDBY",
    STATUS_PANEL_NOT_IN_LIBRARY,	"STATUS_PANEL_NOT_IN_LIBRARY",
				    /* 215 */

    STATUS_PANEL_NOT_FOUND,			"STATUS_PANEL_NOT_FOUND",
    STATUS_STORAGE_CELL_MAP_CHANGED, "STATUS_STORAGE_CELL_MAP_CHANGED",
    STATUS_NOT_A_DRIVE_PANEL,		"STATUS_NOT_A_DRIVE_PANEL",
    STATUS_NO_DRIVES_FOUND,			"STATUS_NO_DRIVES_FOUND",
    STATUS_LKEY_INVALID,			"STATUS_LKEY_INVALID",
			    /* 220 */
    STATUS_LKEY_EXPIRED,			"STATUS_LKEY_EXPIRED",
    STATUS_LKEY_DUE_TO_EXPIRE,			"STATUS_LKEY_DUE_TO_EXPIRE",
    STATUS_LKEY_CAPACITY_EXCEEDED,  		"STATUS_LKEY_CAPACITY_EXCEEDED",
    STATUS_MALFORMED_XML,			"STATUS_MALFORMED_XML",
    STATUS_TAG_NOT_FOUND,			"STATUS_TAG_NOT_FOUND",
                    /* 225 */
    STATUS_CONTENT_NOT_FOUND,			"STATUS_CONTENT_NOT_FOUND",
    STATUS_INVALID_CONTENT,			"STATUS_INVALID_CONTENT",
    STATUS_INCOMPATIBLE_SERVER,			"STATUS_INCOMPATIBLE_SERVER",
    STATUS_MAX_REQUESTS_EXCEEDED,		"STATUS_MAX_REQUESTS_EXCEEDED",
    STATUS_SHARED_MEMORY_ERROR,			"STATUS_SHARED_MEMORY_ERROR",
                    /* 230 */
    STATUS_MGMTCLAS_NOT_FOUND,			"STATUS_MGMTCLAS_NOT_FOUND",


    STATUS_LAST,    				"code %d",
};

static  char    unknown[32];    /* unknown buffer type */

/*
 *      Procedure Type Declarations:
 */
char *
sam_acs_status(
	STATUS status)    /* status code to convert */
{
	int	i;

	/* verify table contents (allow for STATUS_SUCCESS in table size) */
	if ((int)STATUS_LAST !=
	    ((sizeof (status_table) / sizeof (struct st)) - 1))
		sam_syslog(LOG_ERR, "Contents of status table is incorrect");
	else
		for (i = 0; status_table[i].stat_val != STATUS_LAST; i++) {
			if (status == status_table[i].stat_val)
				return (status_table[i].stat_str);
		}

	sprintf(unknown, status_table[i].stat_str, status);
	return (unknown);
}


/*
 * is_storage - is the specified element a storage address
 */
int
is_storage(
	library_t *library,
	const uint_t element)
{
	return (TRUE);
}


/*
 * slot_number - given the element address, return the slot number
 */
uint_t
slot_number(
	library_t *library,
	const uint_t element)
{
	return (element);
}


/*
 * element_address - given the slot number, return the element address
 */
uint_t
element_address(
	library_t *library,
	const uint_t slot)
{
	return (slot);
}
